@@ -36,18 +36,20 @@ struct fake_input {
 	int		uinput;		/* uinput socket */
 	int		rfcomm;		/* RFCOMM socket */
 	uint8_t		ch;		/* RFCOMM channel number */
-	gboolean 	(*connect) (struct input_conn *iconn);
+	gboolean	(*connect) (struct input_conn *iconn, GError **err);
 	int		(*disconnect) (struct input_conn *iconn);
 	void		*priv;
 };
 
-int input_device_register(DBusConnection *conn, const char *path,
-			const bdaddr_t *src, const bdaddr_t *dst, const char *uuid,
-			const uint32_t handle, int timeout);
-int fake_input_register(DBusConnection *conn, const char *path, bdaddr_t *src,
-			bdaddr_t *dst, const char *uuid, uint8_t channel);
+int fake_input_register(DBusConnection *conn, struct btd_device *device,
+			const char *path, bdaddr_t *src, bdaddr_t *dst,
+			const char *uuid, uint8_t channel);
+int input_device_register(DBusConnection *conn, struct btd_device *device,
+			const char *path, const bdaddr_t *src,
+			const bdaddr_t *dst, const char *uuid,
+			uint32_t handle, int timeout);
 int input_device_unregister(const char *path, const char *uuid);
 
-int input_device_set_channel(const bdaddr_t *src, const bdaddr_t *dst, int psm, int nsk);
+int input_device_set_channel(const bdaddr_t *src, const bdaddr_t *dst, int psm,
+							GIOChannel *io);
 int input_device_close_channels(const bdaddr_t *src, const bdaddr_t *dst);
-int input_device_connadd(const bdaddr_t *src, const bdaddr_t *dst);
